clear all
set more off
cd ""
*** Turn the classification table into STATA dta file ***

import excel using "classification.xlsx", firstrow clear
drop id
ren idCoarse id
drop FRBcode description
gen z=strpos(naics, ",")
sum z
local max=r(max)
while `max'>0 {
expand 2 if z>0
bys naics: gen rank=_n if z>0
replace naics=substr(naics, 1, z-1) if rank==1 
replace naics=substr(naics, z+1,.) if rank==2
drop rank
replace z=strpos(naics, ",")
sum z
local max = r(max)
}
replace z = strpos(naics, "-")
gen before=substr(naics,z-1,1) if z>0
gen after=substr(naics,z+1,1) if z>0
destring after before, replace
expand =after-before+1 if z>0
bys naics: gen rank=_n if z>0
gen last_digit=before+rank-1
tostring last_digit, replace
replace naics=substr(naics,1,z-2)+last_digit if z>0
drop in 1/1
drop z-last_digit
sort id
replace naics=strtrim(naics)
compress
save classification, replace

*** Find the code used in 2007 ***

import excel using "Use2007.xlsx", sheet("NAICS codes") cellrange(B7:F643) clear
drop if F==""
drop B E
compress
gen z=strpos(F, ",")
sum z
local max=r(max)
while `max'>0 {
expand 2 if z>0
bys F: gen rank=_n if z>0
replace F=substr(F, 1, z-1) if rank==1 
replace F=substr(F, z+1,.) if rank==2
drop rank
replace z=strpos(F, ",")
sum z
local max = r(max)
}
replace z = strpos(F, "-")
gen before=substr(F,z-1,1) if z>0
gen after=substr(F,z+1,1) if z>0
destring after before, replace
expand =after-before+1 if z>0
bys F: gen rank=_n if z>0
gen last_digit=before+rank-1
tostring last_digit, replace
replace F=substr(F,1,z-2)+last_digit if z>0
drop z-last_digit
replace F=strtrim(F)
replace F=C if F=="n/a"
replace F="5179" if F=="5719" 
gen G=.
gen H=""
merge 1:1 _n using classification
drop _merge
gen length=length(naics)
forvalues x=1/165 {/* the number 165 = _N in classification*/
local naics=naics[`x']
local id=id[`x']
local shortname=shortname[`x']
local length=length[`x']
replace G=`id' if substr(F,1,`length')=="`naics'"
replace H="`shortname'" if substr(F,1,`length')=="`naics'"
}
drop naics-length
rename G id
rename H shortname
rename C ioind
rename D description
rename F naics
bys ioind description id shortname: gen rank=_n
keep if rank==1
drop naics rank
gen appear=1
gen year=2007
save code1997-2007, replace

*** Find the code used in 2002 ***

import excel using "Appendix A_rev 4-24-08.xls", cellrange(B9:D728) clear
drop if D==""
replace B=strtrim(B)
replace C=strtrim(C)
replace B="33712B" if B=="33721A" 
compress
gen z=strpos(D, ",")
sum z
local max=r(max)
while `max'>0 {
expand 2 if z>0
bys D: gen rank=_n if z>0
replace D=substr(D, 1, z-1) if rank==1 
replace D=substr(D, z+1,.) if rank==2
drop rank
replace z=strpos(D, ",")
sum z
local max = r(max)
}
replace z = strpos(D, "-")
gen before=substr(D,z-1,1) if z>0
gen after=substr(D,z+1,1) if z>0
destring after before, replace
expand =after-before+1 if z>0
bys D: gen rank=_n if z>0
gen last_digit=before+rank-1
tostring last_digit, replace
replace D=substr(D,1,z-2)+last_digit if z>0
drop z-last_digit
replace D=strtrim(D)
replace D=B if D=="n.a."
gen G=.
gen H=""
merge 1:1 _n using classification
drop _merge
gen length=length(naics)
forvalues x=1/165 {/* the number 165 = _N in classification*/
local naics=naics[`x']
local id=id[`x']
local shortname=shortname[`x']
local length=length[`x']
replace G=`id' if substr(D,1,`length')=="`naics'"
replace H="`shortname'" if substr(D,1,`length')=="`naics'"
}
drop naics-length
rename G id
rename H shortname
rename B ioind
rename C description
rename D naics

bys ioind description id shortname: gen rank=_n
keep if rank==1
drop naics rank
gen appear=1
gen year=2002
append using code1997-2007
save code1997-2007, replace

*** Find the code used in 1997 ***

import excel using "NAICS-IO.xls", cellrange(A4:J685) clear
drop if C==""
tostring J, replace
replace J="" if J=="."
local counter=0
foreach var of varlist C-J {
local counter=`counter'+1
rename `var' C`counter'
}
reshape long C, i(A B) j(D)
drop if C==""
drop D
replace B=strtrim(B)
replace C=strtrim(C)
compress
gen G=.
gen H=""
merge 1:1 _n using classification
drop _merge
gen length=length(naics) if naics!=""
forvalues x=1/165 {/* the number 165 = _N in classification*/
local naics=naics[`x']
local id=id[`x']
local shortname=shortname[`x']
local length=length[`x']
replace G=`id' if substr(C,1,`length')=="`naics'"
replace H="`shortname'" if substr(C,1,`length')=="`naics'"
}
drop naics-length
rename G id
rename H shortname
rename A ioind
rename B description
rename C naics
bys ioind description id shortname: gen rank=_n
keep if rank==1
drop naics rank
gen appear=1
gen year=1997
append using code1997-2007
save code1997-2007, replace

reshape wide desc id shortname appear, i(ioind) j(year)
*****make sure each io industry is classified in the same index*****
gen test9702 = "error" if id1997!=id2002 & appear1997==1 & appear2002==1
gen test9707 = "error" if id1997!=id2007 & appear1997==1 & appear2007==1
gen test0207 = "error" if id2007!=id2002 & appear2007==1 & appear2002==1
assert test9702==""
assert test9707==""
assert test0207==""
drop test*
*****
local variables "description shortname"
foreach var of local variables {
replace `var'2007=`var'2002 if `var'2007=="" & `var'2002!=""
replace `var'2007=`var'1997 if `var'2007=="" & `var'1997!=""
drop `var'2002 `var'1997
rename `var'2007 `var'
}
replace id2007=id2002 if id2007==. & id2002!=.
replace id2007=id1997 if id2007==. & id1997!=.
drop id2002 id1997
rename id2007 id
save code1997-2007, replace

*** Add the code of final uses ***

import excel using "final uses.xlsx", firstrow clear
append using code1997-2007
order ioind id desc shortname appear*
sort ioind
export excel using corr1997-2007.xlsx, firstrow(variable) replace
