/* This file takes the output of the filter_master m file and produces the two panels of Figures 2-3 of "Accounting for the Sources of Macroeconomic Tail Risks */ 

set scheme s1mono

foreach yy in 01 10 {

insheet using filtered_shocks_eps_`yy'.csv, clear
gen x=_n
gen s_=.
gen k_=.
forvalues x=1/39 {
   sum v`x', d
   replace s_=r(skewness) if `x'==_n
   replace k_=r(kurtosis) if `x'==_n
}
drop v1-v39
keep if _n<40
replace k_=k_-3

if "`yy'"=="01" {

graph twoway (scatter  x s , msymbol(+) mcolor(black)),  legend(off) xtitle("Skewness")  ylab(1 "Mining" 2 "Utilities" 3 "Food Manuf." 4 "Beverage" 5 "Fabric" 6 "Textile" 7 "Apparel" 8 "Leather" 9 "Wood" 10 "Paper" 11 "Printing" 12 "Petroleum" 13 "Chemicals" 14 "Plastics & Rubber" 15 "   Nonmetallic Mineral" 16 "Primary Metal" 17 "Fabricated Metal" 18 "Machinery" 19 "Computer & Electr." 20 "Electrical Equip." 21 "Transportation" 22 "Furniture" 23 "   Misc. Manufacturing" 24 "Publishing" 25 "Agriculture" 26 "Construction" 27 "Wholesale" 28 "Retail" 29 "Transportation" 30 "Information" 31 "FIRE" 32 "Prof. Services" 33 "Education" 34 "Health" 35 "Amusement" 36 "Accommodation" 37 "Food Service" 38 "Other Service" 39 "Government", angle(0) notick labsize(medsmall))  xline(0, lpattern(dash)) ytitle("") fxsize(75) yline(24.5, lpattern(dash)) xlab(-2(1)1)
}

if "`yy'"=="10" {
graph twoway (scatter x s, msymbol(+) mcolor(black)) , legend(off) xtitle("Skewness")  xline(0, lpattern(dash)) ytitle("") ylab("") yline(24.5, lpattern(dash)) xlab(-2(1)1)
}
graph save "skewness_by_industry_eps_`yy'.gph", replace

graph twoway (scatter x k, msymbol(+) mcolor(black)) , legend(off) xtitle("Kurtosis")  xline(0, lpattern(dash)) ytitle("") ylab("") yline(24.5, lpattern(dash)) xlab(0(3)9)
graph save "kurtosis_by_industry_eps_`yy'.gph", replace

if "`yy'"=="01" {
  graph combine skewness_by_industry_eps_`yy'.gph kurtosis_by_industry_eps_`yy'.gph , title(        {&epsilon}=0.1, size(medlarge))
  graph export skew_kurt_figure_2_shocks_eps_`yy'.eps, replace 
  graph export skew_kurt_figure_2_shocks_eps_`yy'.tif, replace width(2300) height(2300)
}

if "`yy'"=="10" {
  graph combine skewness_by_industry_eps_`yy'.gph kurtosis_by_industry_eps_`yy'.gph , title({&epsilon}=1.0, size(medlarge))
  graph export skew_kurt_figure_2_shocks_eps_`yy'.eps, replace 
  graph export skew_kurt_figure_2_shocks_eps_`yy'.tif, replace  width(1700) height(2300)

}


}

insheet using results_table.csv, clear 
ren v1 eps
ren v2 correl
ren v3 coskew
ren v4 cokurt

scatter correl-cokurt eps  , connect(line line line) xtitle("{&epsilon}") ytitle("") color(red blue black) msymbol(Oh T X) legend(lab(1 "Correlation") lab(2 "Co-Skewness") lab(3 "Co-Kurtosis") row(1) region(lwidth(none))) xlab(.1(.1)1.0) ylab(-0.05(0.05)0.25)
graph export skew_kurt_figure_3.eps, replace

ren v5 agg_cor
ren v6 agg_skew
ren v7 agg_kurt

scatter agg* eps , connect(line line line) xtitle("{&epsilon}") ytitle("") color(red blue black) msymbol(Oh T X) legend(lab(1 "Second Moment") lab(2 "Third Moment") lab(3 "Fourth Moment") row(1) region(lwidth(none))) xlab(.1(.1)1.0) ylab(0(0.2)1)

graph export skew_kurt_figure_4.eps, replace
