clear all
set more off
capture log close
log using read_ip, replace

local directory ../tail_risk_
cd "`directory'"
* Part 1. Find the intermediate usage structure

*** Part 1.1. Change the excel correspondence table to STATA .dta files

import excel using "`directory'/Code 1972-1992/corr1972-1992.xlsx", firstrow clear
drop sic description
save corr1972-1992, replace

import excel using "`directory'/Code 1997-2007/corr1997-2007.xlsx", firstrow clear
drop description
save corr1997-2007, replace

import excel using "corrFRB.xlsx", firstrow clear
save corrFRB, replace

import excel using "Classification_Coarse.xlsx", firstrow clear
drop idCoarser
save corrFRB_3_digit, replace

import excel using "`directory'/Code capital flows table/corrCapital.xlsx", firstrow clear
keep code id group rank label
save corrCapital, replace

*** Part 1.2. Import the 1972-1982 IO table

forvalues year=1972(5)2007 {
if `year' == 1972 {
	local fileuse "`directory'/Data source/1972UseMake.txt"
	local filemake "`directory'/Data source/1972UseMake.txt"
	local delim " "
	local correspond "corr1972-1992"
	local varuse "v3"
	local varmake "v4"
	}
if `year' == 1977 | `year'==1982 {
	local fileuse "`directory'/Data source/`year'UseMake.csv"
	local filemake "`directory'/Data source/`year'UseMake.csv"
	local delim ","
	local correspond "corr1972-1992"
	local varuse "v3"
	local varmake "v4"
	}
if `year' == 1987 {
	local fileuse "`directory'/Data source/1987Use.csv"
	local filemake "`directory'/Data source/1987Make.csv"
	local delim ","
	local correspond "corr1972-1992"
	local varuse "v4"
	local varmake "v4"
	}
if `year' == 1992 {
	local fileuse "`directory'/Data source/1992Use.txt"
	local filemake "`directory'/Data source/1992Make.txt"
	local delim " "
	local correspond "corr1972-1992"
	local varuse "v4"
	local varmake "v4"
	}
if `year' == 1997 {
	local fileuse "`directory'/Data source/1997Use.txt"
	local filemake "`directory'/Data source/1997Make.txt"
	local delim ","
	local correspond "corr1997-2007"
	local varuse "v5"
	local varmake "v4"
	}
if `year' == 2002 {
	local fileuse "`directory'/Data source/2002Use.csv"
	local filemake "`directory'/Data source/2002Make.csv"
	local delim ","
	local correspond "corr1997-2007"
	local varuse "v3"
	local varmake "v3"
	}
if `year' == 2007 {
	local fileuse "`directory'/Data source/2007UseMake.csv"
	local filemake "`directory'/Data source/2007UseMake.csv"
	local delim ","
	local correspond "corr1997-2007"
	local varuse "v3"
	local varmake "v4"
	}
	
***** Part 1.2.1 Import the use data

display "**********`year', use**********" 
	/* so that it is trackable in the log*/
import delim using "`fileuse'", delim("`delim'") clear
if `year'==2007 {
	keep if data_set==1
	drop data_set
	reshape long v, i(code) j(v2) string
	replace v2=strupper(v2)
	rename v v3
	rename code v1
	drop if substr(v1,1,1)=="T" | substr(v2,1,1)=="T"
	replace v3=0 if v3==.
	}
capture tostring v1 v2, replace
replace v1=strtrim(v1)
replace v2=strtrim(v2)
replace v1="0"+v1 if `year'<=1992 & length(v1)<6
replace v2="0"+v2 if `year'<=1992 & length(v2)<6
keep v1 v2 `varuse'
rename `varuse' vuse
drop if vuse==0
rename v1 ioind
merge n:1 ioind using `correspond' 
	/* _merge shouldn't be 1 in any observation, check at log*/
drop if _merge==2
drop appear* _merge shortname ioind
rename id c
rename v2 ioind
merge n:1 ioind using `correspond'
	/* _merge shouldn't be 1 in any observation, check at log*/
drop if _merge==2
drop appear* _merge shortname ioind
rename id d
collapse (sum) vuse, by(c d)
drop if c==300 | d==300
	/*scrap, used, incomparable imports and other adjustment*/
save use_temp, replace

***** Part 1.2.2 Import the make data

display "**********`year', make**********"
	/* so that it is trackable in the log*/
import delim using "`filemake'", delim("`delim'") clear
if `year' == 1972 {
	replace v4=0 if v4==.
	foreach var of varlist v5-v21 {
		replace `var'=0 if `var'==.
		replace v4=v4+`var'
		}
	}
if `year' == 2007 {
	keep if data_set==2
	drop data_set
	reshape long v, i(code) j(v2) string
	replace v2=strupper(v2)
	rename v v4
	rename code v1
	drop if substr(v1,1,1)=="T" | substr(v2,1,1)=="T"
	replace v4=0 if v4==.
	}
capture tostring v1 v2, replace
replace v1="0"+v1 if length(v1)<6
replace v2="0"+v2 if length(v2)<6

keep v1 v2 `varmake'
rename `varmake' vmake
drop if vmake==0
rename v1 ioind
merge n:1 ioind using `correspond'
	/* _merge shouldn't be 1 in any observation, check at log*/
drop if _merge==2
drop appear* _merge shortname ioind
rename id s
rename v2 ioind
merge n:1 ioind using `correspond'
	/* _merge shouldn't be 1 in any observation, check at log*/
drop if _merge==2
drop appear* _merge shortname ioind
rename id c
collapse (sum) vmake, by(s c)
bys c: egen total_c = total(vmake)
gen frac = vmake/total_c
drop vmake total_c
drop if c==300 | s==300
	/*scrap, used, incomparable imports and other adjustment*/
save make_temp, replace

******* Part 1.2.3. Combine the use data and make data

display "**********`year', combine**********"
	/* so that it is trackable in the log*/
use make_temp, clear
joinby c using use_temp, unmatched(both)
	/* _merge=2 if commodity code starts with "V". This is the payment to labor,
	   tax and capital*/
replace s=c if _merge==2
replace frac=1 if _merge==2 
replace vuse=vuse*frac
collapse (sum) vuse, by(d s)
ren vuse flows
fillin s d
replace flows=0 if flows==.
drop _fillin
sort s d
gen year=`year'
if `year'!=1972 {
	append using intermediate_flows
	}
save intermediate_flows, replace
}
sort s d
drop if floor(s/100)==4 & floor(d/100)==9
reshape wide flows, i(s d) j(year)
save intermediate_flows, replace

	/* NOTE: the scale in 1982 is 10 time smaller than any other year. So the 
	   output value is 10 times larger in 1982. It is okay if we are only 
	   interested in fractions of intermediate needed from a specific industry.
	   But special attention is required if we want to use the absolute value*/

* Part 2 Produce the MJ, KJ and OJ table

*** Part 2.1. Produce the cost by industry table.

use intermediate_flows, clear
replace s=402 if s==403
	/* Combine taxes (402) and profit (403)*/
replace s=403 if s<300 
	/* Redefine 403 as intermediate cost*/
collapse (sum) flows*, by(s d)
sort d s
drop if d>900
replace flows1972=flows1972[_n-1]*flows1977/(flows1977+flows1977[_n+1]) if s==401
replace flows1972=flows1972[_n-2]*flows1977/(flows1977+flows1977[_n-1]) if s==402
drop if s==400
reshape long flows, i(s d) j(year)
ren d id
gen costAccount = s-400
drop s
reshape wide flows, i(id year) j(costAccount)
gen total_cost=flows1+flows2+flows3
replace flows1=flows1/total_cost
replace flows2=flows2/total_cost
replace flows3=flows3/total_cost
drop total_cost
ren flows1 labor_cost
ren flows2 capital_cost
ren flows3 intermediate_cost
save cost_shares, replace

*** Part 2.2. Produce the consumption table

use intermediate_flows, clear
keep if d==910 | d==929 | d==951
	/* Private consumption, residential investment and gov consumption. */
bys s: egen consumption = sum(flows2007)
gen ratio = flows2007/consumption if d==910
keep if d==910
foreach var of varlist flows1972-flows2007 {
	replace `var' = `var'/ratio
	sum `var'
	replace `var' = `var'/r(sum)
	}
drop consumption ratio d
rename s id
reshape long flows, i(id) j(year)
rename flows consumption_share
save consumption, replace

*** Part 2.3. Use consumption and cost_shares to form data_OJ

use consumption, replace
merge 1:1 id year using cost_shares
	/* Should be a clean merge, check at log */
drop _merge
gen durable=1
replace durable=.125 if (id>=123 & id<=125) | (id>=135 & id<=178) | id==202
	/* Set the depreciation rate of 0.125 for durable goods + construction (B02).
	   Durable goods include wood product (123-125), non-metal mineral product
	   (136-140), metal product (141-152), machine and equipment (153-167),
	   transportation equipment (168-174), furnitures (175,176) and others (177)*/
save dataOJ_ip_expand, replace

use dataOJ_ip_expand, replace
keep if year==1997
drop year id
order labor capital intermediate consumption durable
export delim using "data_OJ1997.csv", delim(",") novarnames replace

*** Part 2.4. Produce dataMJ table

use intermediate_flows, clear
drop if (s>=400 & s<500) | d>900
foreach var of varlist flows* {
   bys d: egen ss=sum(`var')
   replace `var'=`var'/ss
   drop ss
}
reshape long flows , i(s d) j(year)
save dataMJ_ip_expand, replace

keep if year==1997
drop year
reshape wide flows, i(s) j(d)
drop s
export delim using "data_MJ1997.csv",delim(",") novarnames replace

*** Part 2.5. Produce the government investment table

use intermediate_flows, clear
keep if d==952
keep s flows1997
qui sum flows1997
replace flows1997=flows1997/r(sum)
save government_investment, replace


*** Part 2.6. Produce the dataKJ table

import excel using "`directory'/Data source/changes_to_180x123combined.xls",/*
	*/sheet("Changes to 180x123Combined") clear
drop A C I
rename B commodity
foreach var of varlist D-H {
	local nm = `var'[3]
	rename `var' v`nm'
	}
drop in 1/3
drop in 102/105
qui destring v5417-v6230, replace
reshape long v, i(commodity) j(ind)
tostring ind, replace
rename v change
save change_to_capital_table, replace

import excel using "`directory'/Data source/flow1997.xls",/*
	*/sheet("180x123Combined") cellrange(A4:DV184) clear
drop A C
replace B = strtrim(B)
foreach var of varlist B-DV{
local name = `var'[1]
rename `var' v`name'
}
drop in 1/1
qui destring v1110-v813B, replace
rename vCommodity commodity
reshape long v, i(commodity) j(ind) string
merge 1:1 commodity ind using change_to_capital_table
replace v=change if _merge==3 & v!=change
drop change _merge
replace v=0 if v==.

rename commodity code
merge n:1 code using corrCapital
	/* There should be 123 entries for _merge==2. Those are 123 industries*/
drop if _merge==2
drop _merge
drop group-label
rename id s
rename code commodity
rename ind code
merge n:1 code using corrCapital
	/* There should be 180 entries for _merge==2. Those are 180 commodities*/
drop if _merge==2
drop _merge
rename id d
expand = rank if label == "error"
sort code commodity 
by code commodity: gen rank2=_n if label=="error"
replace d = d - rank2 + 1 if label=="error"
replace v = 0.207*v if code=="332B" & d==152
	/* The BEA capital flow table classifies the industries (demand) more coarsely
	   than the FRB G17 table and the classification we adopt. Therefore, for 
	   one industry code in BEA cap table, it might correspond to more than one 
	   industries in our classification. We simply expand the data because we are
	   interested in the relative proportion of capital that comes from other
	   industries. And for simplicity we assume it is the same for all "small
	   industries" with one "large industry". Industry 152 is the only special
	   case b/c part of it belongs to 332B, which is a "large industry" and
	   also parts belonging to 3324 and 332A. The number 0.207 is the portion
	   of GO in the overlapping part relative to the GO of 332B. We multiply the
	   portion to v so that it is addable to the part in 3324 and 332A.*/
keep v s d
drop if s==300
collapse (sum) v, by(s d)
reshape wide v, i(s) j(d)
foreach var of varlist v101-v214 {
	qui sum `var'
	qui replace `var'=`var'/r(sum)
	}
merge 1:1 s using government_investment
foreach var of varlist v101-v214 {
	qui replace `var'=0 if `var'==.
	}
sort s
rename flows1997 v215
drop _merge s
export delim using "data_KJ1997.csv", delim(",") novarnames replace

* Part 3. Produce files using the BEA annual gross output data

*** Part 3.1. The aggregate gross output

import excel using "`directory'/Data source/GDPbyInd_GO_1947-2016.xlsx", /*
	*/sheet("GO") cellrange(C7:BT7) clear
	/* The 7th column is the aggregate economy.*/
xpose, clear
rename v1 GO
gen year = _n+1946
save temp, replace

import excel using "`directory'/Data source/GDPbyInd_GO_1947-2016.xlsx", /*
	*/sheet("ChainPriceIndexes") cellrange(C7:BS7) clear
xpose, clear
rename v1 GOPI
gen year = _n+1946
merge 1:1 year using temp
drop _merge
gen id=1000
order year id GO GOPI
keep if year>=1972 & year<=2016
save aggregate_output, replace

*** Part 3.2. The annual output of non-industrial-production industries

import excel using "`directory'/Data source/GDPbyInd_GO_1947-2016.xlsx", /*
	*/sheet("GO") cellrange(A7:BT104) clear
destring A C-BT, replace force
keep if A==3 | A==11 | A==34 | A==35 | A==40 | A==49 | A==50 | A==54 | A==64 | /*
	*/A==65 | A==75 | A==76 | A==83 | A==87 | A==88 | A==89 | A==90
keep AA - BT
	/* year=1971/2016. Keep one more year before and after so that we can generate
	   the growth rate and weight for all sampling period. */
local counter=1970
foreach var of varlist AA-BT {
	local counter = `counter'+1
	rename `var' v`counter'
	}
gen id = ""
local code "201 202 203 204 205 206 pub 207 rent 208 209 210 211 212 213 214 215"
forvalues x=1/17 {
	replace id = word("`code'", `x') in `x'/`x'
	}
reshape long v, i(id) j(year)
rename v GO
save temp, replace

import excel using "`directory'/Data source/GDPbyInd_GO_1947-2016.xlsx", /*
	*/sheet("ChainPriceIndexes") cellrange(A7:BT104) clear
destring A C-BT, replace force
keep if A==3 | A==11 | A==34 | A==35 | A==40 | A==49 | A==50 | A==54 | A==64 | /*
	*/A==65 | A==75 | A==76 | A==83 | A==87 | A==88 | A==89 | A==90
	/* nonip industries*/
keep AA - BT
	/* year=1971/2016. Keep one more year before and after so that we can generate
	   the growth rate and weight for all sampling period. */
local counter=1970
foreach var of varlist AA-BT {
	local counter = `counter'+1
	rename `var' v`counter'
	}
gen id = ""
local code "201 202 203 204 205 206 pub 207 rent 208 209 210 211 212 213 214 215"
forvalues x=1/17 {
	replace id = word("`code'", `x') in `x'/`x'
	}
reshape long v, i(id) j(year)
rename v GOPI

merge 1:1 id year using temp
drop _merge
gen quantity = GO/GOPI
sort year id

replace GOPI = (GO-GO[_n+10])/(quantity-quantity[_n+10]) if id=="206"|id=="207"
replace GO = GO - GO[_n+10] if id=="206" | id=="207"
replace GOPI = (GO+GO[_n+9])/(quantity+quantity[_n+9]) if id=="208"
replace GO = GO + GO[_n+9] if id=="208"
	/* These are Info-publishing, FIRE-rent&lease, and rent&lease+PROF.*/
		   
drop if id=="pub" | id=="rent"
destring id, replace
sort year id
by year: egen nonip_quant=sum(GO)
gen share = GO/nonip_quant

expand 4
sort id year
by id year: gen qtr=_n
gen share2=.
by id: replace share2=5/8*share+3/8*share[_n-4] if qtr==1
by id: replace share2=7/8*share+1/8*share[_n-4] if qtr==2
by id: replace share2=7/8*share+1/8*share[_n+4] if qtr==3
by id: replace share2=5/8*share+3/8*share[_n+4] if qtr==4
	/* We assume that the year-average share equals to the share at the middle
	   of the year. For example, share in 1972 is the share at July 1st of 1972.
	   Besides, the share changes linealy from one mid-point to the next.
	   Therefore, the share in each quarter is weighted average of two mid-
	   points before and behind. The FRB data provides only the qtr GO of the
	   whole non-ip group. We use these shares to uncover the qtr GO of each
	   industry. */
drop share 
rename share2 share
keep if year>=1972 & year<=2016
sort year qtr id
by year qtr: egen s1=sum(share)
replace share=share/s1
drop s1
keep id year qtr share
save nonip_share, replace

* Part 4. Produce files using the FRB data

*** Part 4.1. The ipweight table
import delim using "`directory'/Data source/ipweights_sa.txt", delim("  ") clear
keep v1
save temp, replace 
	/*This is to generate the proper FRB code*/
import delim using "`directory'/Data source/ipweights_sa.txt", delim(";") clear
gen year = substr(v1,16,4)
forvalues x=1/12 { 
	gen month`x' = substr(v1,10*`x'+15,8)
	replace month`x'=strtrim(month`x')
	destring month`x', replace force
	}
drop v1
destring year, replace force
merge 1:1 _n using temp
drop _merge
rename v1 FRB_code
drop if year==.
compress
keep FRB_code year month2 month5 month8 month11
keep if year>=1972 & year<=2016
merge n:1 FRB_code using corrFRB_3_digit

drop _merge
order id FRB_code year month*
drop FRB_code
rename month2 weight1
rename month5 weight2
rename month8 weight3
rename month11 weight4
drop if idCoarse==.
collapse (sum) weight* , by(idCoarse yea)
reshape long weight, i(idCoarse year) j(qtr)
bys year qtr: egen s1=sum(weight)
replace weight=weight/s1
drop s1
collapse (sum) weight, by(idCoarse year qtr)
sort id year qtr
gen share=.
append using nonip_share
replace idCoarse=id if idCoarse==.
save weights_temp, replace
collapse (sum) weight share, by(year qtr idCoarse)
save weights_temp_coarse, replace

use weights_temp_coarse, replace
keep if idCoarse<200
sort id year qtr
bys id: gen x=_n
drop year qtr
reshape wide weight, i(id) j(x)
save weights_temp_wide, replace

cd "`directory'"
import delim using "`directory'/Data source/FRB_G17_.csv", delim(",") clear
keep if seriesname == "gdp" |seriesname =="ip"|seriesname =="nonip"
drop descriptions-unique
xpose, clear
rename v1 gdp
rename v2 nonip
rename v3 ip
drop in 1/1
gen year = 1972+floor((_n-1)/4)
gen qtr=mod(_n,4)
replace qtr=4 if qtr==0
replace nonip=nonip/gdp
replace ip=ip/gdp
keep if year>=1972 & year<=2016
keep nonip ip year qtr
merge 1:n year qtr using weights_temp_coarse
sort year qtr idCoarse
gen share2 = share*nonip
gen weight2=weight*ip
drop nonip ip share weight _merge
gen weight = max(share2, weight2)
drop share2 weight2
reshape wide weight, i(year qtr) j(idCoarse)
export delim using "weights.csv", replace
 
*** Part 4.2. The FRB G17 table
import delim using "`directory'/Data source/FRB_G17_.csv", delim(",") clear
drop if substr(seriesname, -3,.)=="N.Q"
keep if _n>=33 & _n<=255
	/* Those dropped are not seasonally adjusted data, major industry groups 
	   (coarse), market group, capital, capital utilization etc. Those remains
	   are seasonally adjusted industries at a proper detail level.*/
replace seriesname=substr(seriesname, 4, strpos(seriesname,".S.Q")-4)
drop descriptions-uniqueidentifier
rename seriesname FRB_code
compress
merge 1:1 FRB_code using corrFRB_3_digit
order id FRB_code
keep if _merge==3
drop _merge FRB_code
rename q1 v7
rename q2 v8
rename q3 v9
rename q4 v10

reshape long v, i(idCoarse) j(year_qtr)
ren idCoarse id
gen year=floor((year_qtr-7)/4)+1972
gen qtr=mod(year_qtr-6,4)
replace qtr=4 if qtr==0

sort id year_qtr
drop year_qtr
by id: gen growth_qtr = ln(v/v[_n-1])
rename v GO_qtr
keep if year>=1972 & year<=2016
drop if id==.
save group_A, replace

import delim using "`directory'/Data source/FRB_G17_.csv", delim(",") clear
keep if seriesname=="nonip"
drop descriptions-seriesname
xpose, clear
gen year=floor((_n-1)/4)+1972
gen qtr=mod(_n,4)
replace qtr=4 if qtr==0
rename v1 groupGO_FRB_qtr
keep if year>=1972 & year<=2016
merge 1:n year qtr using nonip_share
	/* Should be a perfect merge. Check at log.*/
drop _merge
order id year qtr groupGO* share
sort id year qtr
gen GO_qtr = groupGO_FRB_qtr*share
by id: gen growth_qtr=ln(GO_qtr/GO_qtr[_n-1])
	/* The quarterly data of FRB do not add up to the annual data of BEA.
      	However, the does not affect the computation of growth rate,
	   because the number in each qtr is magnified by the same scale. */
keep id year qtr GO_qtr growth_qtr
save group_B, replace

import delim using "`directory'/Data source/FRB_G17_.csv", delim(",") clear
keep if seriesname=="gdp"
drop descriptions-seriesname
xpose, clear
gen year=floor((_n-1)/4)+1972
gen qtr=mod(_n,4)
replace qtr=4 if qtr==0
rename v1 GO_qtr

keep if year>=1972 & year<=2016
gen id = 999
sort year qtr
gen growth_qtr=ln(GO_qtr/GO_qtr[_n-1])

append using group_A
append using group_B

gen year_qtr=yq(year,qtr)
drop year qtr
xtset id year_qtr

by id: egen mean_growth=mean(growth_qtr)
gen cycle_linear=growth_qtr-mean_growth
keep id year_qtr cycle*
save ioJ_data, replace

reshape wide cycle_linear, i(year_qtr) j(id)
replace year_qtr=_n-1
drop in 1/1 
drop if cycle_linear215==.
export delim using "ioJ_data_linear.csv", delim(",") novarnames nolabel replace

